@@ -26,7 +26,7 @@
  * accordingly.
  */
 
-#define G_LOG_DOMAIN "GUPnP-ContextManager-Linux"
+#define G_LOG_DOMAIN "gupnp-context-manager"
 
 #include <config.h>
 
@@ -126,10 +126,8 @@ dump_rta_attr (sa_family_t family, struct rtattr *rt_attr)
                 case IFA_BROADCAST: label = "IFA_BROADCAST"; break;
                 case IFA_ANYCAST: label = "IFA_ANYCAST"; break;
                 case IFA_CACHEINFO: label = "IFA_CACHEINFO"; break;
-                case IFA_MULTICAST:
-                        label = "IFA_MULTICAST";
-                        break;
-#if defined(HAVE_IFA_FLAGS)
+                case IFA_MULTICAST: label = "IFA_MULTICAST"; break;
+#if defined(IFA_FLAGS)
                 case IFA_FLAGS: label = "IFA_FLAGS"; break;
 #endif
                 default: label = "Unknown"; break;
